/**
 * @file webpack.base.conf 生产 & 开发环境通用配置
 */
'use strict'
const path = require('path')
const webpack = require('webpack')
const createVueLoaderOptions = require('./vue-loader.config')

// 拼接路径
function resolve (track) {
    return path.join(__dirname, '..', track)
}

// 当前运行环境
const isDev = process.env.NODE_ENV === 'development'

const defaultPlugins = [
    // 主要作用是在此处可以根据isdev配置process.env,一是可以在js代码中可以获取到process.env
    // 二是webpack或则vue等根据process.env如果是development,会给一些特殊的错误提醒等,而这些特殊项在正式环境是不需要的
    new webpack.DefinePlugin({
        'process.env': {
            NODE_ENV: `"${process.env.NODE_ENV}"`
        }
    })
]

module.exports = {
    entry: path.resolve(__dirname, '../src/index.js'),

    resolve: {
        // 自动解析文件扩展名(补全文件后缀)(左 -> 右)
        extensions: ['.js', '.vue', '.json'],
        // 配置别名映射
        alias: {
            // 键后加上$,表示精准匹配
            vue$: 'vue/dist/vue.esm.js',
            '@': resolve('src'),
            utils: resolve('src/public/utils/utils'),
            components: resolve('src/components'),
            public: resolve('public')
        }
    },

    module: {
        rules: [
            {
                test: /\.(vue|js|jsx)$/,
                loader: 'eslint-loader',
                exclude: /node_modules/,
                enforce: 'pre' // 先使用eslint预处理对应的文件, 校验通过再使用对应的loader处理
            },
            {
                test: /\.vue$/,
                loader: 'vue-loader', // 处理.vue文件
                options: createVueLoaderOptions(isDev)
            },
            {
                test: /\.jsx$/,
                loader: 'babel-loader' // 处理jsx文件
            },
            // 使用babel-loader来处理src下所有js文件, 详细babel配置在.babelrc
            {
                test: /\.js$/,
                use: {
                    loader: 'babel-loader'
                },
                include: resolve('src')
            },
            // 使用url-loader(file-loader的一个再封装)对引入的图片进行编码,此处可将小于20480字节(20kb)的图片转为DataURL(base64),
            // 大于limit字节的会调用file-loader进行处理
            // 图片一般发布后都是长缓存,故此处文件名加入hash做版本区分
            {
                test: /\.(jpg|png|gif|svg|jpeg|txt|exl)$/,
                use: [
                    {
                        loader: 'url-loader',
                        options: {
                            limit: 1024,
                            name: '[name]_[hash:8].[ext]',
                            outputPath: 'images/' // 静态资源输出路径
                        }
                    }
                ]
            },
            {
                // iconfont字体文件
                test: /\.(eot|ttf|svg|woff|woff2)(\?\S*)?$/,
                // 处理test中的文件类型需要用到的loader类型(名称)
                use: {
                    loader: 'file-loader'
                }
            }
        ]
    },

    plugins: defaultPlugins.concat([])
}
